{

  Exemplo de Discagem Automatica
  ==============================

  Este exemplo visa demonstrar as configuracoes basicas da placa
  FXO VB0408 para discar uma sequencia de numeros de uma lista. Quando h
  detecao de atendimento, inicia a reproduao de uma frase.

  A discagem aqui utiliza as funes primitivas PickUp, Dial, HangUp, alm de
  esperar os eventos OnDialToneDetected, OnBusyDetected etc.

  Consulte tambm o exemplo MakeCall que mostra o funcionamento do mtodo
  MakeCall que facilita a tarefa de discagem.

}
unit fDiscador;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OleCtrls,ShellApi, voicerlibLib_TLB, StdCtrls, Grids, ExtCtrls;

type
  TForm1 = class(TForm)
    memLog: TMemo;
    tmrDisca: TTimer;
    GroupBox1: TGroupBox;
    GroupBox3: TGroupBox;
    cmdStart: TButton;
    cmdShutdown: TButton;
    cmdIniciarDisc: TButton;
    cmdPararDisc: TButton;
    vlib: Tvoicerlib;
    Panel1: TPanel;
    grdNumeros: TStringGrid;
    Label1: TLabel;
    Label2: TLabel;
    txtNumero: TEdit;
    cmdAdicionar: TButton;
    cmdRemover: TButton;
    Button2: TButton;
    Panel2: TPanel;
    Label4: TLabel;
    lblNumPortas: TLabel;
    Label3: TLabel;
    txtPorta: TEdit;
    LeiaMe: TButton;
    Image1: TImage;
    btnLimpaLog: TButton;
    procedure cmdStartClick(Sender: TObject);
    procedure cmdShutdownClick(Sender: TObject);
    procedure cmdIniciarDiscClick(Sender: TObject);
    procedure cmdAdicionarClick(Sender: TObject);
    procedure cmdRemoverClick(Sender: TObject);
    procedure cmdPararDiscClick(Sender: TObject);
    procedure grdNumerosSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure vLibAfterDial(ASender: TObject; Port: Smallint);
    procedure vLibPlayStop(ASender: TObject; Port, StopStatus: Smallint);
    procedure tmrDiscaTimer(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure vlibDialToneDetected(ASender: TObject; Port: Smallint);
    procedure vlibCalling(ASender: TObject; Port: Smallint);
    procedure btnLimpaLogClick(Sender: TObject);
    procedure LeiaMeClick(Sender: TObject);
    procedure vlibBusyDetected(ASender: TObject; Port: Smallint);
    procedure vlibAfterPickup(ASender: TObject; Port: Smallint);
    procedure vlibAnswerDetected(ASender: TObject; Port,
      AnswerType: Smallint);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
    procedure GeraLog(nPorta: Integer; sMsg: String);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  nSelRow: Integer;
  sReturn    : array [0..255] of Char;
  nContToques : array [1..81] of Integer;

implementation

{$R *.dfm}

procedure TForm1.cmdStartClick(Sender: TObject);
var
  nRetornoVoicerLib,nPorta,nRet: Integer;
begin
  try
    vLib.ConfigPath := 'C:\Arquivos de programas\VoicerLib4';
    GeraLog(0, 'Iniciando Voicerlib...Aguarde...');
    //Inicializa a voicerlib
    nRetornoVoicerLib := vLib.StartVoicerLib;
    //Trata o retorno da inicializacao
    vlib.ReturnCodeToString(nRetornoVoicerLib,Cardinal(@sReturn));

    if nRetornoVoicerLib <> DG_EXIT_SUCCESS then
    begin
      GeraLog(0,'Erro ao inicializar a VoicerLib. Erro -> ' + sReturn);
      Application.Terminate;
    end;

    lblNumPortas.Caption := IntToStr(vlib.GetPortsCount);

  except
    Application.MessageBox('cmdStartClick', 'Except', mb_OK);
  end;

  GeraLog(0, 'Placa Inicializada com sucesso!');
  cmdShutdown.Enabled := true;
  cmdStart.Enabled := false;

  for nPorta:=1 to vlib.GetPortsCount do
  begin
  //Cria tratamento de supervisao de linha
  nRet := vlib.CreateCallProgress(nPorta,'cp_default.cfg');

    case nRet of
      DG_ERROR_DRIVER_CLOSED : GeraLog(nPorta,'CreateCallProgress -Driver iniciado');
      DG_ERROR_THREAD_ALREADY_RUNNING : GeraLog(nPorta,'CreateCallProgress - A thread do callprogress j havia sido iniciada anteriormente');
      DG_ERROR_COULD_NOT_CREATE_THREAD : GeraLog(nPorta,'CreateCallProgress - Falha na inicializao da thread do callprogress');
    end;
  end;
end;

procedure TForm1.cmdShutdownClick(Sender: TObject);
var
 nPorta,nRet : Integer;
begin

  for nPorta := 1 to vlib.GetPortsCount do
  begin
    //Desabilita supervisao de linha
    vlib.DisableCallProgress(nPorta);
    // Finaliza o tratamento de supervisao de linha
    nRet := vlib.DestroyCallProgress(nPorta);
    case nRet of
      DG_ERROR_DRIVER_CLOSED : GeraLog(nPorta,'DisableCallProgress - Driver desabilitado');
      DG_ERROR_PORT_OUT_OF_RANGE : GeraLog(nPorta,'DisableCallProgress - Parmetro porta fora do intervalo permitido, excedendo o nmero de portas instaladas');
    end;
  end;

  //Rotina que finaliza a VoicerLib
  nRet := vLib.ShutdownVoicerLib;
  if nRet <> DG_EXIT_SUCCESS then
    GeraLog(0, 'Erro ao finalizar a VoicerLib')
  else
    GeraLog(0, 'VoicerLib finalizada com sucesso');

  cmdStart.Enabled := true;
  cmdShutdown.Enabled := false;
end;

procedure TForm1.cmdIniciarDiscClick(Sender: TObject);
var
  i : integer;
begin
  try
    for i:=1 to vlib.GetPortsCount do
      nContToques[i] := 0;  //contador de numero de toques

    //atende a porta
    vLib.PickUp(StrToInt(txtPorta.text), 100);
    GeraLog(StrToInt(txtPorta.text), 'Atendeu a porta ');
  except
    Application.MessageBox('cmdIniciarDiscClick', 'Except', mb_OK);
  end;
end;

procedure TForm1.cmdAdicionarClick(Sender: TObject);
var
  i: Integer;
begin
  try
    if grdNumeros.Cells[1, nSelRow] = '' then
    begin
      grdNumeros.Cells[1, nSelRow] := '0';
      grdNumeros.Cells[0, nSelRow] := txtNumero.Text;
      Exit;
    end;

    for i := 9 downto 1 do
    begin
      grdNumeros.Cells[0, i] := grdNumeros.Cells[0, i - 1];
      grdNumeros.Cells[1, i] := grdNumeros.Cells[1, i - 1];
    end;

    grdNumeros.Cells[1, 0] := '0';
    grdNumeros.Cells[0, 0] := txtNumero.Text;
  except
    Application.MessageBox('cmdAdicionarClick', 'Except', mb_OK);
  end;
end;

procedure TForm1.cmdRemoverClick(Sender: TObject);
begin
  try
    grdNumeros.Cells[0, nSelRow] := '';
    grdNumeros.Cells[1, nSelRow] := '';
  except
    Application.MessageBox('cmdRemoverClick', 'Except', mb_OK);
  end;
end;

procedure TForm1.cmdPararDiscClick(Sender: TObject);
begin
  try
    //para parar a discagem desligamos a porta
    if vlib.IsPlaying(StrToInt(txtPorta.text)) then
      Vlib.StopPlayFile(StrToInt(txtPorta.text));
    vLib.HangUp(StrToInt(txtPorta.text));
    GeraLog(StrToInt(txtPorta.text), 'Desligou');
    tmrDisca.Enabled := False;
  except
    Application.MessageBox('cmdPararDiscClick', 'Except', mb_OK);
  end;
end;

procedure TForm1.grdNumerosSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  try
    nSelRow := aRow;
  except
    Application.MessageBox('grdNumerosSelectCell', 'Except', mb_OK);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  cVersao : array[0..255] of Char;
  nRet : Integer;

begin
  try
    nSelRow := 0;
    //Verifica a versao da dll e ocx da voicerlib
    nRet := vlib.GetLibVersion(Cardinal(@cVersao),0);
    if nRet = DG_EXIT_SUCCESS then
      GeraLog(0,'Verso DLL -> ' + cVersao)
    else
      GeraLog(0,'Verso DLL -> Falha');

    nRet := vlib.GetLibVersion(Cardinal(@cVersao),1);
    if nRet = DG_EXIT_SUCCESS then
      GeraLog(0,'Verso OCX -> ' + cVersao)
    else
      GeraLog(0,'Verso OCX -> Falha');

  except
    Application.MessageBox('FormCreate', 'Except', mb_OK);
  end;
end;

procedure TForm1.GeraLog(nPorta: Integer; sMsg: String);
begin
  try
    memLog.Lines.Add('<' + TimeToStr(Time) + '>' + IntToStr(nPorta) + ' -> ' + sMsg);
  except;
    Application.MessageBox('GeraLog', 'Except', mb_OK);
  end;
end;

procedure TForm1.vLibAfterDial(ASender: TObject; Port: Smallint);
var
  nRet1:Integer;
begin

  GeraLog(Port, 'Discou com sucesso'); 
  //Habilita supervisao de linha apos a Discagem
  nRet1 := vlib.EnableCallProgress(Port,CP_ENABLE_ALL);
  case nRet1 of

    DG_EXIT_SUCCESS : GeraLog(Port,'Habilitou CallProgress - Executado com sucesso');
    DG_ERROR_DRIVER_CLOSED : GeraLog(Port,'CallProgress - Driver Desabilitado');
    DG_ERROR_PORT_OUT_OF_RANGE : GeraLog(Port,'CallProgress - Parametro porta fora do intervalo permitido, excedendo o numero de portas instaladas');
    DG_ERROR_PARAM_OUTOFRANGE : GeraLog(Port,'CallProgress - Parametro CPType fora do intervalo permitido');
    DG_ERROR_THREAD_NOT_RUNNING : GeraLog(Port,'CallProgress - Thread de controle no foi iniciada pelo CreateCallProgress');
  end;

  //Habilita deteccao de atendimento
  vlib.EnableAnswerDetection(Port);

end;

procedure TForm1.vLibPlayStop(ASender: TObject; Port,
  StopStatus: Smallint);
begin
  try
    GeraLog(Port, 'Reproduo Finalizada');
    vlib.HangUp(Port);
    //Habilita timer para esperar proximo numero
    tmrDisca.Enabled := true;
    GeraLog(Port, 'Prximo nmero...');
  except;
    Application.MessageBox('vLibPlayStop', 'Except', mb_OK);
  end;
end;

procedure TForm1.tmrDiscaTimer(Sender: TObject);
begin
  try
    nContToques[StrToInt(txtPorta.text)] := 0;
    tmrDisca.Enabled := False;
    vlib.PickUp(StrToInt(txtPorta.text),100);
    GeraLog(StrToInt(txtPorta.text), 'Vai discar novo numero...');
  except;
    Application.MessageBox('tmrDiscaTimer', 'Except', mb_OK);
  end;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  //Verifica se o driver esta inicializado
  if vlib.DriverEnabled then
  begin
    Application.Messagebox('Finalize a VoicerLib antes de fechar a aplicao','Aviso',MB_OK + MB_ICONSTOP);
    CanClose := false;
  end;
end;

procedure TForm1.vlibDialToneDetected(ASender: TObject; Port: Smallint);
var
  i,nRet: Integer;
  bAchou: boolean;
begin
  //apos atender a porta e receber a confirmao de ocupao
  //vamos discar o numero que esta na lista
  try
    GeraLog(Port, 'DialToneDetected');
    bAchou := false;
    for i := 0 to 9 do
    begin
      if grdNumeros.Cells[1, i] = '0' then
      begin
        bAchou := true;
        grdNumeros.Cells[1, i] := '1';
        //Executa discagem
        nRet := vLib.Dial(Port, grdNumeros.Cells[0, i], 100, dtTone);
        if nRet <> DG_EXIT_SUCCESS then
          GeraLog(Port,'Erro ao discar'); 

        GeraLog(Port, 'Dial: ' + grdNumeros.Cells[0, i]);
        Break;
      end;
    end;

    if not bAchou then
    begin
      vlib.HangUp(Port);
      GeraLog(Port, '**** Fim dos Nmeros ****');
    end;

    nContToques[Port] := 0;

  except
    Application.MessageBox('vLibDialToneDetected', 'Except', mb_OK);
  end;

end;

procedure TForm1.vlibCalling(ASender: TObject; Port: Smallint);
begin
  Inc(nContToques[Port]);
  GeraLog(Port,'Chamando... ' + IntToStr(nContToques[Port]) + 'o. toque');
  if (nContToques[Port] >= 5) then
  begin
    vlib.DisableAnswerDetection(Port);
     
    GeraLog(Port,'No atendeu.' + IntToStr(nContToques[Port]) + 'o. toque');
    nContToques[Port] := 0;

    //Se estiver reproduzindo, manda parar
    if vlib.IsPlaying(Port) Then
      vlib.StopPlayFile(Port);

    vlib.HangUp(Port);
    tmrDisca.Enabled := true;

  end;
end;

procedure TForm1.btnLimpaLogClick(Sender: TObject);
begin
  memLog.Clear;
end;

procedure TForm1.LeiaMeClick(Sender: TObject);
var
  L: String;
begin
  L:= ExtractFilePath(Application.ExeName) + 'Leiame.txt' ;
  ShellExecute(0, nil ,PChar(L), nil,nil,SW_NORMAL);
end;

procedure TForm1.vlibBusyDetected(ASender: TObject; Port: Smallint);
begin
  GeraLog(Port,'Ocupado...');
  vlib.DisableAnswerDetection(Port);
  //Se estiver reproduzindo, manda parar
  if vlib.IsPlaying(Port) Then
    vlib.StopPlayFile(Port);

  vlib.HangUp(Port);
  tmrDisca.Enabled := true;
end;

procedure TForm1.vlibAfterPickup(ASender: TObject; Port: Smallint);
var
  nRet1 : Integer;
begin

  //Habilita supervisao de linha apos atendimento
  nRet1 := vlib.EnableCallProgress(Port,CP_ENABLE_LINETONE_OR_BUSY);
  case nRet1 of

    DG_EXIT_SUCCESS : GeraLog(Port,'Habilitou CallProgress - Executado com sucesso');
    DG_ERROR_DRIVER_CLOSED : GeraLog(Port,'CallProgress - Driver Desabilitado');
    DG_ERROR_PORT_OUT_OF_RANGE : GeraLog(Port,'CallProgress - Parametro porta fora do intervalo permitido, excedendo o numero de portas instaladas');
    DG_ERROR_PARAM_OUTOFRANGE : GeraLog(Port,'CallProgress - Parametro CPType fora do intervalo permitido');
    DG_ERROR_THREAD_NOT_RUNNING : GeraLog(Port,'CallProgress - Thread de controle no foi iniciada pelo CreateCallProgress');

  end;
end;

procedure TForm1.vlibAnswerDetected(ASender: TObject; Port,
  AnswerType: Smallint);
var
  nRet1:Integer;
begin

  if AnswerType = AUDIO_DETECTED Then
    GeraLog(Port, 'Detectou atendimento por **AUDIO**')
  else
    GeraLog(Port, 'Detectou atendimento por **TIMEOUT**');

  //Desabilita deteccao de atendimento
  vlib.DisableAnswerDetection(Port);
  //Habilita supervisao de linha na deteccao de atendimento
  nRet1 := vlib.EnableCallProgress(Port,CP_ENABLE_BUSY_OR_FAX);

  case nRet1 of
    DG_EXIT_SUCCESS : GeraLog(Port,'Executado com sucesso o callprogress na deteccao de atendimento');
    DG_ERROR_DRIVER_CLOSED : GeraLog(Port,'CallProgress - Driver Desabilitado');
    DG_ERROR_PORT_OUT_OF_RANGE : GeraLog(Port,'CallProgress - Parametro porta fora do intervalo permitido, excedendo o numero de portas instaladas');
    DG_ERROR_PARAM_OUTOFRANGE : GeraLog(Port,'CallProgress - Parametro CPType fora do intervalo permitido');
    DG_ERROR_THREAD_NOT_RUNNING : GeraLog(Port,'CallProgress - Thread de controle no foi iniciada pelo CreateCallProgress');
  end;
  //Seta formato de reproducao
  vLib.SetPlayFormat(Port, ffWaveALaw);//seta o formato do arquivo para a porta 1

  //Reproduz uma frase
  nRet1 := vLib.PlayFile(Port, ExtractFilePath(Application.ExeName) + '\Teste1.wav', '', 0); //reproduz a frase
  if nRet1 = DG_EXIT_SUCCESS then
    GeraLog(Port, 'Reproduzindo mensagem....')
  else
    GeraLog(Port, 'Erro ao reproduzir mensagem');

end;

procedure TForm1.Button2Click(Sender: TObject);
var
  i : Integer;
begin
  for i := 10 downto 1 do
  begin
    grdNumeros.Cells[0, i-1] := '';
    grdNumeros.Cells[1, i-1] := '';
  end;
end;

end.













